@@ -101,6 +101,31 @@ def has_reference(dtype):
         has_ref = has_reference(basedt)
     return has_ref
 
+def is_vlen(dtype):
+    if dtype.metadata and 'vlen' in dtype.metadata:
+        return True
+    else:
+        return False
+
+def get_fillvalue(dset):
+
+    fillvalue = None
+    if not is_vlen(dset.dtype):
+        try:
+            # can trigger a runtime error if fillvalue is undefined
+            fillvalue = dset.fillvalue
+        except RuntimeError:
+            pass  # ignore
+    return fillvalue
+
+def is_compact(dset):
+    if isinstance(dset.id.id, str):
+        return False  # compact storage not used with HSDS
+    cpl = dset.id.get_create_plist()
+    if cpl.get_layout() == h5py.h5d.COMPACT:
+        return True
+    else:
+        return False
 
 def convert_dtype(srcdt, ctx):
     """ Return a dtype based on input dtype, converting any Reference types from
@@ -301,22 +326,9 @@ def create_dataset(dobj, ctx):
     fout = ctx["fout"]
 
     chunks = None
+    dset = None
 
-    if dobj.dtype.metadata and 'vlen' in dobj.dtype.metadata:
-        is_vlen = True
-    else:
-        is_vlen = False
-
-    fillvalue = None
-    if not is_vlen:
-        try:
-            # can trigger a runtime error if fillvalue is undefined
-            fillvalue = dobj.fillvalue
-        except RuntimeError:
-            pass  # ignore
-
-    # can_use_storeinfo = use_storeinfo(dobj, ctx)
-    if ctx["dataload"] == "link" and not is_vlen:
+    if ctx["dataload"] == "link" and not is_vlen(dobj.dtype) and not is_compact(dobj):
         dset_dims = dobj.shape
         logging.debug("dset_dims: {}".format(dset_dims))
         rank = len(dset_dims)
@@ -390,7 +402,7 @@ def create_dataset(dobj, ctx):
             anon_dset = fout.create_dataset(None, shape=chunkinfo_arr_dims, dtype=dt)
             anon_dset[...] = chunkinfo_arr
             logging.debug("anon_dset: {}".format(anon_dset))
-            logging.debug("annon_values: {}".format(anon_dset[...]))
+            #logging.debug("anon_values: {}".format(anon_dset[...]))
             chunks["class"] = 'H5D_CHUNKED_REF_INDIRECT'
             chunks["file_uri"] = ctx["s3path"]
             chunks["dims"] = dobj.chunks
@@ -403,7 +415,7 @@ def create_dataset(dobj, ctx):
 
     try:
         tgt_dtype = convert_dtype(dobj.dtype, ctx)
-        if len(dobj.shape) == 0 or (is_vlen and is_h5py(fout)):
+        if len(dobj.shape) == 0 or (is_vlen(dobj.dtype) and is_h5py(fout)):
             # don't use compression/chunks for scalar datasets
             # or vlen
             compression = None
@@ -426,8 +438,7 @@ def create_dataset(dobj, ctx):
             maxshape = dobj.maxshape
             scaleoffset = dobj.scaleoffset
 
-        if is_vlen:
-            fillvalue=None
+        fillvalue=get_fillvalue(dobj)
         dset = fout.create_dataset(
             dobj.name, shape=dobj.shape, dtype=tgt_dtype, chunks=chunks,
             compression=compression, shuffle=shuffle, maxshape=maxshape,
@@ -475,19 +486,8 @@ def write_dataset(src, tgt, ctx):
         tgt[()] = x
         return
 
-    if src.dtype.metadata and 'vlen' in src.dtype.metadata:
-        is_vlen = True
-    else:
-        is_vlen = False
 
-
-    fillvalue = None
-    if not is_vlen:
-        try:
-            # can trigger a runtime error if fillvalue is undefined
-            fillvalue = src.fillvalue
-        except RuntimeError:
-            pass  # ignore
+    fillvalue = get_fillvalue(src)
 
     msg = "iterating over chunks for {}".format(src.name)
     logging.info(msg)
@@ -642,18 +642,20 @@ def load_file(fin, fout, verbose=False, dataload="ingest", s3path=None, compress
     ctx["fin"] = fin
     ctx["fout"] = fout
     ctx["verbose"] = verbose
-    ctx["dataload"] = dataload  # ingest, s3link, None
+    ctx["dataload"] = dataload  # ingest, link, or None
     ctx["default_compression"] = compression
     ctx["default_compression_opts"] = compression_opts
     ctx["s3path"] = s3path
     ctx["srcid_desobj_map"] = {}
 
     def copy_attribute_helper(name, obj):
+        logging.info(f"copy attribute - name: {name}  obj: {obj.name}")
         tgt = fout[name]
         for a in obj.attrs:
             copy_attribute(tgt, a, obj, ctx)
 
     def object_create_helper(name, obj):
+        logging.info(f"object create helper - name: {name} obj: {obj.name}")
         class_name = obj.__class__.__name__
         if class_name in ("Dataset", "Table"):
             create_dataset(obj, ctx)
@@ -664,7 +666,7 @@ def load_file(fin, fout, verbose=False, dataload="ingest", s3path=None, compress
 
     def object_link_helper(name, obj):
         class_name = obj.__class__.__name__
-        logging.debug("object_link_helper for object: {}".format(obj.name))
+        logging.info("object_link_helper for object: {}".format(obj.name))
         if class_name == "Group":
             # create any soft/external links
             fout = ctx["fout"]
@@ -676,11 +678,7 @@ def load_file(fin, fout, verbose=False, dataload="ingest", s3path=None, compress
         logging.debug("object_copy_helper for object: {}".format(obj.name))
 
         if class_name in ("Dataset", "Table"):
-            if obj.dtype.metadata and 'vlen' in obj.dtype.metadata:
-                is_vlen = True
-            else:
-                is_vlen = False
-            if ctx["dataload"] == "link" and not is_vlen:
+            if ctx["dataload"] == "link" and not is_vlen(obj.dtype) and not is_compact(obj):
                 logging.info("skip datacopy for link reference")
             else:
                 logging.debug("calling write_dataset for dataset: {}".format(obj.name))
@@ -710,7 +708,7 @@ def load_file(fin, fout, verbose=False, dataload="ingest", s3path=None, compress
     create_links(fin, fout, ctx)  # create root soft/external links
     fin.visititems(object_link_helper)
 
-    if dataload == "ingest":
+    if dataload == "ingest" or dataload == "link":
         # copy dataset data
         logging.info("copying dataset data")
         fin.visititems(object_copy_helper)
