@@ -90,7 +90,7 @@ class File(Group):
         return self._limits
 
     def __init__(self, domain, mode=None, endpoint=None, username=None, password=None, bucket=None,
-        api_key=None, use_session=True, use_cache=True, logger=None, owner=None, linked_domain=None, retries=3, **kwds):
+        api_key=None, use_session=True, use_cache=True, logger=None, owner=None, linked_domain=None, retries=10, **kwds):
         """Create a new file object.
 
         See the h5py user guide for a detailed explanation of the options.
@@ -127,6 +127,7 @@ class File(Group):
         """
 
         groupid = None
+        dn_ids = []
         # if we're passed a GroupId as domain, just initialize the file object
         # with that.  This will be faster and enable the File object to share the same http connection.
         if mode is None and endpoint is None and username is None \
@@ -193,45 +194,6 @@ class File(Group):
                 elif "hs_bucket" in cfg:
                     bucket = cfg["hs_bucket"]
 
-            if api_key is None:
-                if "HS_API_KEY" in os.environ:
-                    api_key = os.environ["HS_API_KEY"]
-                elif "hs_api_key" in cfg:
-                    api_key = cfg["hs_api_key"]
-
-            if not api_key:
-                # if Azure AD ids are set, pass them to HttpConn via api_key dict
-
-                ad_app_id = None  # Azure AD HSDS Server id
-                if "HS_AD_APP_ID" in os.environ:
-                    ad_app_id = os.environ["HS_AD_APP_ID"]
-                elif "hs_ad_app_id" in cfg:
-                    ad_app_id = cfg["hs_ad_app_id"]
-
-                ad_tenant_id = None # Azure AD tenant id
-                if "HS_AD_TENANT_ID" in os.environ:
-                    ad_tenant_id = os.environ["HS_AD_TENANT_ID"]
-                elif "hs_ad_tenant_id" in cfg:
-                    ad_tenant_id = cfg["hs_ad_tenant_id"]
-
-                ad_resource_id = None # Azure AD resource id
-                if "HS_AD_RESOURCE_ID" in os.environ:
-                    ad_resource_id = os.environ["HS_AD_RESOURCE_ID"]
-                elif "hs_ad_resource_id" in cfg:
-                    ad_resource_id = cfg["hs_ad_resource_id"]
-
-                ad_client_secret = None # Azure client secret
-                if "HS_AD_CLIENT_SECRET" in os.environ:
-                    ad_client_secret = os.environ["HS_AD_CLIENT_SECRET"]
-                elif "hs_ad_client_secret" in cfg:
-                    ad_client_secret = cfg["hs_ad_client_secret"]
-
-                if ad_app_id and ad_tenant_id and ad_resource_id:
-                    # contruct dict to pass to HttpConn
-                    api_key = {"AD_APP_ID": ad_app_id, "AD_TENANT_ID": ad_tenant_id, "AD_RESOURCE_ID": ad_resource_id}
-                    if ad_client_secret:
-                        api_key["AD_CLIENT_SECRET"] = ad_client_secret
-
             http_conn =  HttpConn(domain, endpoint=endpoint,
                     username=username, password=password, bucket=bucket, mode=mode,
                     api_key=api_key, use_session=use_session, use_cache=use_cache, logger=logger, retries=retries)
@@ -240,7 +202,7 @@ class File(Group):
 
             # try to do a GET from the domain
             req = "/"
-            params = {}
+            params =  {"getdnids": 1} # return dn ids if available
             if use_cache and mode == 'r':
                 params["getobjs"] = "T"
                 params["include_attrs"] = "T"
@@ -262,7 +224,7 @@ class File(Group):
             if rsp.status_code == 200 and mode == 'w':
                 # delete existing domain
                 rsp = http_conn.DELETE(req, params=params)
-                if rsp.status_code != 200:
+                if rsp.status_code not in (200, 410):
                     # failed to delete
                     http_conn.close()
                     raise IOError(rsp.status_code, rsp.reason)
@@ -280,7 +242,7 @@ class File(Group):
                     body["owner"] = owner
                 if linked_domain:
                     body["linked_domain"] = linked_domain
-                rsp = http_conn.PUT(req, body=body)
+                rsp = http_conn.PUT(req, params=params, body=body)
                 if rsp.status_code != 201:
                     http_conn.close()
                     raise IOError(rsp.status_code, rsp.reason)
@@ -291,6 +253,9 @@ class File(Group):
                 http_conn.close()
                 raise IOError(404, "Unexpected error")
 
+            if "dn_ids" in root_json:
+                dn_ids = root_json["dn_ids"] 
+
             root_uuid = root_json['root']
 
             if "limits" in root_json:
@@ -347,7 +312,8 @@ class File(Group):
         self._id = groupid
         self._verboseInfo = None  # aditional state we'll get when requested
         self._verboseUpdated = None # when the verbose data was fetched
-        self._lastScan = None  # when summary stats where last updated o server
+        self._lastScan = None  # when summary stats where last updated by server
+        self._dn_ids = dn_ids
 
 
         Group.__init__(self, self._id)
@@ -545,29 +511,33 @@ class File(Group):
         """  Tells the service to complete any pending updates to permanent storage
         """
         self.log.debug("flush")
-        if  self._id.id.startswith("g-"):
+        if  self.mode == "r+" and self._id.id.startswith("g-"):
             # Currently flush only works with HSDS
             self.log.info("sending PUT flush request")
             req = '/'
-            body = {"flush": 1}
-            self.PUT(req, body=body)
+            body = {"flush": 1, "getdnids": 1}
+            rsp = self.PUT(req, body=body)
+            if "dn_ids" in rsp:
+                dn_ids = rsp["dn_ids"]
+                orig_ids = set(self._dn_ids)
+                current_ids = set(dn_ids)
+                self._dn_ids = current_ids
+                if orig_ids and orig_ids != current_ids:
+                    self.log.debug("original dn_ids: {}".format(orig_ids))
+                    self.log.debug("current dn_ids: {}".format(current_ids))
+                    self.log.warn("HSDS nodes have changed")
+                    raise IOError(500, "Unexpected Error")
             self.log.info("PUT flush complete")
 
-    def close(self):
+    def close(self, flush=False):
         """ Clears reference to remote resource.
         """
         # this will close the socket of the http_conn singleton
 
         self.log.debug("close, mode: {}".format(self.mode))
-        # do a PUT flush if this file is writable and the server is HSDS
-        if  self.mode == "r+" and self._id.id.startswith("g-"):
-            # Currently flush only works with HSDS
-            self.log.info("sending PUT flush request")
-            req = '/'
-            body = {"flush": 1}
-            self.PUT(req, body=body)
-
-
+        # do a PUT flush if this file is writable and the server is HSDS and flush is set
+        if flush:
+            self.flush()
         if self._id._http_conn:
             self._id._http_conn.close()
         self._id.close()
